@@ -22,7 +22,6 @@ from oslo_log import log
 from manila.common import constants
 from manila import exception
 from manila.i18n import _
-from manila.i18n import _LW
 from manila.share import api as share_api
 import manila.utils as utils
 
@@ -48,12 +47,14 @@ CONF.register_opts(migration_opts)
 
 class ShareMigrationHelper(object):
 
-    def __init__(self, context, db, share):
+    def __init__(self, context, db, share, access_helper):
 
         self.db = db
         self.share = share
         self.context = context
+        self.access_helper = access_helper
         self.api = share_api.API()
+        self.access_helper = access_helper
 
         self.migration_create_delete_share_timeout = (
             CONF.migration_create_delete_share_timeout)
@@ -84,11 +85,12 @@ class ShareMigrationHelper(object):
             else:
                 time.sleep(tries ** 2)
 
-    def create_instance_and_wait(self, share, share_instance, host):
+    def create_instance_and_wait(self, share, dest_host, new_share_network_id,
+                                 new_az_id, new_share_type_id):
 
         new_share_instance = self.api.create_instance(
-            self.context, share, share_instance['share_network_id'],
-            host['host'])
+            self.context, share, new_share_network_id, dest_host,
+            new_az_id, share_type_id=new_share_type_id)
 
         # Wait for new_share_instance to become ready
         starttime = time.time()
@@ -103,14 +105,14 @@ class ShareMigrationHelper(object):
                 msg = _("Failed to create new share instance"
                         " (from %(share_id)s) on "
                         "destination host %(host_name)s") % {
-                    'share_id': share['id'], 'host_name': host['host']}
+                    'share_id': share['id'], 'host_name': dest_host}
                 self.cleanup_new_instance(new_share_instance)
                 raise exception.ShareMigrationFailed(reason=msg)
             elif now > deadline:
                 msg = _("Timeout creating new share instance "
                         "(from %(share_id)s) on "
                         "destination host %(host_name)s") % {
-                    'share_id': share['id'], 'host_name': host['host']}
+                    'share_id': share['id'], 'host_name': dest_host}
                 self.cleanup_new_instance(new_share_instance)
                 raise exception.ShareMigrationFailed(reason=msg)
             else:
@@ -128,74 +130,55 @@ class ShareMigrationHelper(object):
         try:
             self.delete_instance_and_wait(new_instance)
         except Exception:
-            LOG.warning(_LW("Failed to cleanup new instance during generic"
-                        " migration for share %s."), self.share['id'])
+            LOG.warning("Failed to cleanup new instance during generic"
+                        " migration for share %s.", self.share['id'])
+
+    def cleanup_access_rules(self, share_instance, share_server):
 
-    def cleanup_access_rules(self, share_instance, share_server, driver):
         try:
-            self.revert_access_rules(share_instance, share_server, driver)
+            self.revert_access_rules(share_instance, share_server)
         except Exception:
-            LOG.warning(_LW("Failed to cleanup access rules during generic"
-                        " migration for share %s."), self.share['id'])
+            LOG.warning("Failed to cleanup access rules during generic"
+                        " migration for share %s.", self.share['id'])
 
-    def change_to_read_only(self, share_instance, share_server,
-                            readonly_support, driver):
+    def revert_access_rules(self, share_instance, share_server):
 
-        # NOTE(ganso): If the share does not allow readonly mode we
-        # should remove all access rules and prevent any access
+        # Cast all rules to 'queued_to_apply' so that they can be re-applied.
+        updates = {'state': constants.ACCESS_STATE_QUEUED_TO_APPLY}
+        self.access_helper.get_and_update_share_instance_access_rules(
+            self.context, updates=updates,
+            share_instance_id=share_instance['id'])
 
-        rules = self.db.share_access_get_all_for_instance(
-            self.context, share_instance['id'])
+        self.access_helper.update_access_rules(
+            self.context, share_instance['id'], share_server=share_server)
 
-        if len(rules) > 0:
-
-            if readonly_support:
-
-                LOG.debug("Changing all of share %s access rules "
-                          "to read-only.", self.share['id'])
-
-                for rule in rules:
-                    rule['access_level'] = 'ro'
-
-                driver.update_access(self.context, share_instance, rules,
-                                     add_rules=[], delete_rules=[],
-                                     share_server=share_server)
-            else:
-
-                LOG.debug("Removing all access rules for migration of "
-                          "share %s." % self.share['id'])
-
-                driver.update_access(self.context, share_instance, [],
-                                     add_rules=[], delete_rules=rules,
-                                     share_server=share_server)
-
-    def revert_access_rules(self, share_instance, share_server, driver):
-
-        rules = self.db.share_access_get_all_for_instance(
-            self.context, share_instance['id'])
-
-        if len(rules) > 0:
-            LOG.debug("Restoring all of share %s access rules according to "
-                      "DB.", self.share['id'])
-
-            driver.update_access(self.context, share_instance, rules,
-                                 add_rules=[], delete_rules=[],
-                                 share_server=share_server)
+        utils.wait_for_access_update(
+            self.context, self.db, share_instance,
+            self.migration_wait_access_rules_timeout)
 
     def apply_new_access_rules(self, new_share_instance):
 
-        self.db.share_instance_access_copy(self.context, self.share['id'],
-                                           new_share_instance['id'])
+        rules = self.db.share_instance_access_copy(
+            self.context, self.share['id'], new_share_instance['id'])
 
-        rules = self.db.share_access_get_all_for_instance(
-            self.context, new_share_instance['id'])
+        if rules:
+            self.api.allow_access_to_instance(self.context, new_share_instance)
 
-        if len(rules) > 0:
-            LOG.debug("Restoring all of share %s access rules according to "
-                      "DB.", self.share['id'])
-
-            self.api.allow_access_to_instance(self.context, new_share_instance,
-                                              rules)
             utils.wait_for_access_update(
                 self.context, self.db, new_share_instance,
                 self.migration_wait_access_rules_timeout)
+        else:
+            LOG.debug("No access rules to sync to destination share instance.")
+
+    @utils.retry(exception.ShareServerNotReady, retries=8)
+    def wait_for_share_server(self, share_server_id):
+        share_server = self.db.share_server_get(self.context, share_server_id)
+        if share_server['status'] == constants.STATUS_ERROR:
+            raise exception.ShareServerNotCreated(
+                share_server_id=share_server_id)
+        elif share_server['status'] == constants.STATUS_ACTIVE:
+            return share_server
+        else:
+            raise exception.ShareServerNotReady(
+                share_server_id=share_server_id, time=511,
+                state=constants.STATUS_AVAILABLE)
