@@ -71,6 +71,10 @@ MTU = 1234
 DEFAULT_MTU = 1500
 MANILA_HOST_NAME = '%(host)s@%(backend)s#%(pool)s' % {
     'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME}
+QOS_EXTRA_SPEC = 'netapp:maxiops'
+QOS_SIZE_DEPENDENT_EXTRA_SPEC = 'netapp:maxbpspergib'
+QOS_NORMALIZED_SPEC = 'maxiops'
+QOS_POLICY_GROUP_NAME = 'fake_qos_policy_group_name'
 
 CLIENT_KWARGS = {
     'username': 'admin',
@@ -97,6 +101,7 @@ SHARE = {
     },
     'replica_state': constants.REPLICA_STATE_ACTIVE,
     'status': constants.STATUS_AVAILABLE,
+    'share_server': None,
 }
 
 FLEXVOL_TO_MANAGE = {
@@ -105,7 +110,19 @@ FLEXVOL_TO_MANAGE = {
     'name': FLEXVOL_NAME,
     'type': 'rw',
     'style': 'flex',
-    'size': '1610612736',  # rounds down to 1 GB
+    'size': '1610612736',  # rounds up to 2 GB
+}
+
+FLEXVOL_WITHOUT_QOS = copy.deepcopy(FLEXVOL_TO_MANAGE)
+FLEXVOL_WITHOUT_QOS.update({'qos-policy-group-name': None})
+FLEXVOL_WITH_QOS = copy.deepcopy(FLEXVOL_TO_MANAGE)
+FLEXVOL_WITH_QOS.update({'qos-policy-group-name': QOS_POLICY_GROUP_NAME})
+
+QOS_POLICY_GROUP = {
+    'policy-group': QOS_POLICY_GROUP_NAME,
+    'vserver': VSERVER1,
+    'max-throughput': '3000iops',
+    'num-workloads': 1,
 }
 
 EXTRA_SPEC = {
@@ -120,6 +137,18 @@ EXTRA_SPEC = {
     'netapp_raid_type': 'raid4',
 }
 
+EXTRA_SPEC_WITH_QOS = copy.deepcopy(EXTRA_SPEC)
+EXTRA_SPEC_WITH_QOS.update({
+    'qos': True,
+    QOS_EXTRA_SPEC: '3000',
+})
+
+EXTRA_SPEC_WITH_SIZE_DEPENDENT_QOS = copy.deepcopy(EXTRA_SPEC)
+EXTRA_SPEC_WITH_SIZE_DEPENDENT_QOS.update({
+    'qos': True,
+    QOS_SIZE_DEPENDENT_EXTRA_SPEC: '1000',
+})
+
 PROVISIONING_OPTIONS = {
     'thin_provisioned': True,
     'snapshot_policy': 'default',
@@ -130,6 +159,10 @@ PROVISIONING_OPTIONS = {
     'split': True,
 }
 
+PROVISIONING_OPTIONS_WITH_QOS = copy.deepcopy(PROVISIONING_OPTIONS)
+PROVISIONING_OPTIONS_WITH_QOS.update(
+    {'qos_policy_group': QOS_POLICY_GROUP_NAME})
+
 PROVISIONING_OPTIONS_BOOLEAN = {
     'thin_provisioned': True,
     'dedup_enabled': False,
@@ -230,6 +263,7 @@ USER_NETWORK_ALLOCATIONS = [
         'network_type': 'vlan',
         'label': 'user',
         'mtu': MTU,
+        'gateway': '10.10.10.1',
     },
     {
         'id': '7eabdeed-bad2-46ea-bd0f-a33884c869e0',
@@ -239,6 +273,30 @@ USER_NETWORK_ALLOCATIONS = [
         'network_type': 'vlan',
         'label': 'user',
         'mtu': MTU,
+        'gateway': '10.10.10.1',
+    }
+]
+
+USER_NETWORK_ALLOCATIONS_IPV6 = [
+    {
+        'id': '234dbb10-9a36-46f2-8d89-3d909830c356',
+        'ip_address': 'fd68:1a09:66ab:8d51:0:10:0:1',
+        'cidr': 'fd68:1a09:66ab:8d51::/64',
+        'segmentation_id': '2000',
+        'network_type': 'vlan',
+        'label': 'user',
+        'mtu': MTU,
+        'gateway': 'fd68:1a09:66ab:8d51:0:0:0:1',
+    },
+    {
+        'id': '6677deed-bad2-46ea-bd0f-a33884c869e0',
+        'ip_address': 'fd68:1a09:66ab:8d51:0:10:0:2',
+        'cidr': 'fd68:1a09:66ab:8d51::/64',
+        'segmentation_id': '2000',
+        'network_type': 'vlan',
+        'label': 'user',
+        'mtu': MTU,
+        'gateway': 'fd68:1a09:66ab:8d51:0:0:0:1',
     }
 ]
 
@@ -251,6 +309,7 @@ ADMIN_NETWORK_ALLOCATIONS = [
         'network_type': 'flat',
         'label': 'admin',
         'mtu': MTU,
+        'gateway': '10.10.20.1'
     },
 ]
 
@@ -326,7 +385,7 @@ SHARE_FOR_CG1 = {
         'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME},
     'name': 'share_1',
     'share_proto': 'NFS',
-    'source_cgsnapshot_member_id': None,
+    'source_share_group_snapshot_member_id': None,
 }
 
 SHARE_FOR_CG2 = {
@@ -335,7 +394,7 @@ SHARE_FOR_CG2 = {
         'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME},
     'name': 'share_2',
     'share_proto': 'NFS',
-    'source_cgsnapshot_member_id': None,
+    'source_share_group_snapshot_member_id': None,
 }
 
 # Clone dest of SHARE_FOR_CG1
@@ -345,7 +404,7 @@ SHARE_FOR_CG3 = {
         'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME},
     'name': 'share3',
     'share_proto': 'NFS',
-    'source_cgsnapshot_member_id': CG_SNAPSHOT_MEMBER_ID1,
+    'source_share_group_snapshot_member_id': CG_SNAPSHOT_MEMBER_ID1,
 }
 
 # Clone dest of SHARE_FOR_CG2
@@ -355,7 +414,7 @@ SHARE_FOR_CG4 = {
         'host': HOST_NAME, 'backend': BACKEND_NAME, 'pool': POOL_NAME},
     'name': 'share4',
     'share_proto': 'NFS',
-    'source_cgsnapshot_member_id': CG_SNAPSHOT_MEMBER_ID2,
+    'source_share_group_snapshot_member_id': CG_SNAPSHOT_MEMBER_ID2,
 }
 
 EMPTY_CONSISTENCY_GROUP = {
@@ -403,9 +462,10 @@ CG_SNAPSHOT_MEMBER_2 = {
 }
 
 CG_SNAPSHOT = {
-    'cgsnapshot_members': [CG_SNAPSHOT_MEMBER_1, CG_SNAPSHOT_MEMBER_2],
+    'share_group_snapshot_members': [CG_SNAPSHOT_MEMBER_1,
+                                     CG_SNAPSHOT_MEMBER_2],
     'share_group': CONSISTENCY_GROUP,
-    'consistency_group_id': CONSISTENCY_GROUP_ID,
+    'share_group_id': CONSISTENCY_GROUP_ID,
     'id': CG_SNAPSHOT_ID,
     'project_id': TENANT_ID,
 }
@@ -570,6 +630,7 @@ POOLS = [
         'snapshot_support': True,
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
+        'qos': True,
     },
     {
         'pool_name': AGGREGATES[1],
@@ -591,6 +652,7 @@ POOLS = [
         'snapshot_support': True,
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
+        'qos': True,
     },
 ]
 
@@ -612,6 +674,7 @@ POOLS_VSERVER_CREDS = [
         'snapshot_support': True,
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
+        'qos': False,
     },
     {
         'pool_name': AGGREGATES[1],
@@ -630,6 +693,7 @@ POOLS_VSERVER_CREDS = [
         'snapshot_support': True,
         'create_share_from_snapshot_support': True,
         'revert_to_snapshot_support': True,
+        'qos': False,
     },
 ]
 
@@ -1183,8 +1247,7 @@ EXPANDED_PROCESSOR_COUNTERS = [
 
 def get_config_cmode():
     config = na_fakes.create_configuration_cmode()
-    config.local_conf.set_override('share_backend_name', BACKEND_NAME,
-                                   enforce_type=True)
+    config.local_conf.set_override('share_backend_name', BACKEND_NAME)
     config.reserved_share_percentage = 5
     config.netapp_login = CLIENT_KWARGS['username']
     config.netapp_password = CLIENT_KWARGS['password']
@@ -1200,3 +1263,11 @@ def get_config_cmode():
     config.netapp_volume_snapshot_reserve_percent = 8
     config.netapp_vserver = VSERVER1
     return config
+
+
+def get_network_info(user_network_allocation, admin_network_allocation):
+    net_info = copy.deepcopy(NETWORK_INFO)
+    net_info['network_allocations'] = user_network_allocation
+    net_info['admin_network_allocations'] = admin_network_allocation
+
+    return net_info
