@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import itertools
 
 import ddt
+import itertools
 from tempest import config
 from tempest.lib import exceptions as lib_exc
 import testtools
@@ -77,7 +77,7 @@ def _create_delete_ro_access_rule(self, version):
 
 
 @ddt.ddt
-class ShareIpRulesForNFSTest(base.BaseSharesMixedTest):
+class ShareIpRulesForNFSTest(base.BaseSharesTest):
     protocol = "nfs"
 
     @classmethod
@@ -87,23 +87,23 @@ class ShareIpRulesForNFSTest(base.BaseSharesMixedTest):
                 cls.protocol not in CONF.share.enable_ip_rules_for_protocols):
             msg = "IP rule tests for %s protocol are disabled" % cls.protocol
             raise cls.skipException(msg)
-        # create share type
-        cls.share_type = cls._create_share_type()
-        cls.share_type_id = cls.share_type['id']
-
-        # create share
-        cls.share = cls.create_share(cls.protocol,
-                                     share_type_id=cls.share_type_id)
+        cls.share = cls.create_share(cls.protocol)
         cls.access_type = "ip"
         cls.access_to = "2.2.2.2"
 
     @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
-    @ddt.data(*set(['1.0', '2.9', '2.27', '2.28', LATEST_MICROVERSION]))
-    def test_create_delete_access_rules_with_one_ip(self, version):
-
-        # test data
-        access_to = "1.1.1.1"
+    @ddt.data(*itertools.chain(
+        itertools.product({'1.0', '2.9', '2.37', LATEST_MICROVERSION}, {4}),
+        itertools.product({'2.38', LATEST_MICROVERSION}, {6})
+    ))
+    @ddt.unpack
+    def test_create_delete_access_rules_with_one_ip(self, version,
+                                                    ip_version):
 
+        if ip_version == 4:
+            access_to = utils.rand_ip()
+        else:
+            access_to = utils.rand_ipv6_ip()
         # create rule
         if utils.is_microversion_eq(version, '1.0'):
             rule = self.shares_client.create_access_rule(
@@ -146,12 +146,16 @@ class ShareIpRulesForNFSTest(base.BaseSharesMixedTest):
                 rule_id=rule["id"], share_id=self.share['id'], version=version)
 
     @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
-    @ddt.data(*set(['1.0', '2.9', '2.27', '2.28', LATEST_MICROVERSION]))
-    def test_create_delete_access_rule_with_cidr(self, version):
-
-        # test data
-        access_to = "1.2.3.4/32"
-
+    @ddt.data(*itertools.chain(
+        itertools.product({'1.0', '2.9', '2.37', LATEST_MICROVERSION}, {4}),
+        itertools.product({'2.38', LATEST_MICROVERSION}, {6})
+    ))
+    @ddt.unpack
+    def test_create_delete_access_rule_with_cidr(self, version, ip_version):
+        if ip_version == 4:
+            access_to = utils.rand_ip(network=True)
+        else:
+            access_to = utils.rand_ipv6_ip(network=True)
         # create rule
         if utils.is_microversion_eq(version, '1.0'):
             rule = self.shares_client.create_access_rule(
@@ -216,7 +220,7 @@ class ShareIpRulesForCIFSTest(ShareIpRulesForNFSTest):
 
 
 @ddt.ddt
-class ShareUserRulesForNFSTest(base.BaseSharesMixedTest):
+class ShareUserRulesForNFSTest(base.BaseSharesTest):
     protocol = "nfs"
 
     @classmethod
@@ -227,14 +231,7 @@ class ShareUserRulesForNFSTest(base.BaseSharesMixedTest):
                 CONF.share.enable_user_rules_for_protocols):
             msg = "USER rule tests for %s protocol are disabled" % cls.protocol
             raise cls.skipException(msg)
-        # create share type
-        cls.share_type = cls._create_share_type()
-        cls.share_type_id = cls.share_type['id']
-
-        # create share
-        cls.share = cls.create_share(cls.protocol,
-                                     share_type_id=cls.share_type_id)
-
+        cls.share = cls.create_share(cls.protocol)
         cls.access_type = "user"
         cls.access_to = CONF.share.username_for_user_rules
 
@@ -306,7 +303,7 @@ class ShareUserRulesForCIFSTest(ShareUserRulesForNFSTest):
 
 
 @ddt.ddt
-class ShareCertRulesForGLUSTERFSTest(base.BaseSharesMixedTest):
+class ShareCertRulesForGLUSTERFSTest(base.BaseSharesTest):
     protocol = "glusterfs"
 
     @classmethod
@@ -317,14 +314,7 @@ class ShareCertRulesForGLUSTERFSTest(base.BaseSharesMixedTest):
                 CONF.share.enable_cert_rules_for_protocols):
             msg = "Cert rule tests for %s protocol are disabled" % cls.protocol
             raise cls.skipException(msg)
-        # create share type
-        cls.share_type = cls._create_share_type()
-        cls.share_type_id = cls.share_type['id']
-
-        # create share
-        cls.share = cls.create_share(cls.protocol,
-                                     share_type_id=cls.share_type_id)
-
+        cls.share = cls.create_share(cls.protocol)
         cls.access_type = "cert"
         # Provide access to a client identified by a common name (CN) of the
         # certificate that it possesses.
@@ -422,7 +412,7 @@ class ShareCertRulesForGLUSTERFSTest(base.BaseSharesMixedTest):
 
 
 @ddt.ddt
-class ShareCephxRulesForCephFSTest(base.BaseSharesMixedTest):
+class ShareCephxRulesForCephFSTest(base.BaseSharesTest):
     protocol = "cephfs"
 
     @classmethod
@@ -434,14 +424,7 @@ class ShareCephxRulesForCephFSTest(base.BaseSharesMixedTest):
             msg = ("Cephx rule tests for %s protocol are disabled." %
                    cls.protocol)
             raise cls.skipException(msg)
-        # create share type
-        cls.share_type = cls._create_share_type()
-        cls.share_type_id = cls.share_type['id']
-
-        # create share
-        cls.share = cls.create_share(cls.protocol,
-                                     share_type_id=cls.share_type_id)
-
+        cls.share = cls.create_share(cls.protocol)
         cls.access_type = "cephx"
         # Provide access to a client identified by a cephx auth id.
         cls.access_to = "bob"
@@ -449,13 +432,15 @@ class ShareCephxRulesForCephFSTest(base.BaseSharesMixedTest):
     @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
     @ddt.data(*itertools.product(
         set(['2.13', '2.27', '2.28', LATEST_MICROVERSION]),
-        ("alice", "alice_bob", "alice bob")))
+        ("alice", "alice_bob", "alice bob"),
+        ('rw', 'ro')))
     @ddt.unpack
-    def test_create_delete_cephx_rule(self, version, access_to):
+    def test_create_delete_cephx_rule(self, version, access_to, access_level):
         rule = self.shares_v2_client.create_access_rule(
-            self.share["id"], self.access_type, access_to, version=version)
+            self.share["id"], self.access_type, access_to, version=version,
+            access_level=access_level)
 
-        self.assertEqual('rw', rule['access_level'])
+        self.assertEqual(access_level, rule['access_level'])
         for key in ('deleted', 'deleted_at', 'instance_mappings'):
             self.assertNotIn(key, rule.keys())
         self.shares_v2_client.wait_for_access_rule_status(
@@ -468,7 +453,7 @@ class ShareCephxRulesForCephFSTest(base.BaseSharesMixedTest):
 
 
 @ddt.ddt
-class ShareRulesTest(base.BaseSharesMixedTest):
+class ShareRulesTest(base.BaseSharesTest):
 
     @classmethod
     def resource_setup(cls):
@@ -500,9 +485,7 @@ class ShareRulesTest(base.BaseSharesMixedTest):
             cls.access_type = "cephx"
             cls.access_to = "eve"
         cls.shares_v2_client.share_protocol = cls.protocol
-        cls.share_type = cls._create_share_type()
-        cls.share_type_id = cls.share_type['id']
-        cls.share = cls.create_share(share_type_id=cls.share_type_id)
+        cls.share = cls.create_share()
 
     @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
     @ddt.data(*set(['1.0', '2.9', '2.27', '2.28', LATEST_MICROVERSION]))
@@ -521,6 +504,15 @@ class ShareRulesTest(base.BaseSharesMixedTest):
             rule = self.shares_v2_client.create_access_rule(
                 self.share["id"], self.access_type, self.access_to,
                 version=version)
+
+        # verify added rule keys since 2.33 when create rule
+        if utils.is_microversion_ge(version, '2.33'):
+            self.assertIn('created_at', list(rule.keys()))
+            self.assertIn('updated_at', list(rule.keys()))
+        else:
+            self.assertNotIn('created_at', list(rule.keys()))
+            self.assertNotIn('updated_at', list(rule.keys()))
+
         # rules must start out in 'new' until 2.28 & 'queued_to_apply' after
         if utils.is_microversion_le(version, "2.27"):
             self.assertEqual("new", rule['state'])
@@ -549,6 +541,8 @@ class ShareRulesTest(base.BaseSharesMixedTest):
         keys = ("id", "access_type", "access_to", "access_level")
         if utils.is_microversion_ge(version, '2.21'):
             keys += ("access_key", )
+        if utils.is_microversion_ge(version, '2.33'):
+            keys += ("created_at", "updated_at", )
         for key in keys:
             [self.assertIn(key, r.keys()) for r in rules]
         for key in ('deleted', 'deleted_at', 'instance_mappings'):
@@ -589,7 +583,7 @@ class ShareRulesTest(base.BaseSharesMixedTest):
             raise self.skipException(msg)
 
         # create share
-        share = self.create_share(share_type_id=self.share_type_id)
+        share = self.create_share()
 
         # create rule
         if utils.is_microversion_eq(version, '1.0'):
